import re


def pad_msid(msid):
    "zerofill string for article_id value"
    return "{:05d}".format(int(msid))


def file_extension(file_name):
    "parse file extension from file name"
    if not file_name:
        return None
    return file_name.rsplit(".", 1)[-1]


# match ascii characters from decimal 0 to 31, as hexidecimal character entitiy strings
# e.g. &#x001D; or &#x01;
CONTROL_CHARACTER_ENTITY_MATCH_PATTERN = r"&#x0{0,2}[0-1][0-9A-Fa-f];"
# string to replace character entities with
CONTROL_CHARACTER_ENTITY_REPLACEMENT = "_____"


def match_control_character_entities(string):
    "search the string for character entities of XML-incompatible control characters"
    match_pattern = re.compile(CONTROL_CHARACTER_ENTITY_MATCH_PATTERN)
    return match_pattern.findall(string)


def replace_control_character_entities(string):
    "replace character entities of control characters in the string"
    match_pattern = re.compile(CONTROL_CHARACTER_ENTITY_MATCH_PATTERN)
    return match_pattern.sub(CONTROL_CHARACTER_ENTITY_REPLACEMENT, string)


def match_control_characters(string):
    "search the string for XML-incompatible control characters"
    # char 9 is newline, 10 is tab, 13 is carriage return
    allowed = [9, 10, 13]
    return [char for char in string[:] if ord(char) <= 31 and ord(char) not in allowed]


def replace_control_characters(string):
    "replace control characters in the string"
    for char in list(set(match_control_characters(string))):
        string = string.replace(char, CONTROL_CHARACTER_ENTITY_REPLACEMENT)
    return string


def xlink_href(tag):
    "return the xlink:href attribute of the tag"
    return tag.get("{http://www.w3.org/1999/xlink}href")


def open_tag(tag_name, attr=None):
    "XML string for an open tag"
    if not attr:
        return "<%s>" % tag_name
    attr_values = []
    for name, value in sorted(attr.items()):
        attr_values.append('%s="%s"' % (name, value))
    return "<%s %s>" % (tag_name, " ".join(attr_values))


def close_tag(tag_name):
    "XML string for a close tag"
    return "</%s>" % tag_name


NAMESPACE_MAP = {
    "xmlns:mml": "http://www.w3.org/1998/Math/MathML",
    "xmlns:xlink": "http://www.w3.org/1999/xlink",
}


def namespace_string():
    "return a string of XML namespaces"
    return " ".join(
        [
            '%s="%s"' % (attrib_name, attrib_value)
            for attrib_name, attrib_value in NAMESPACE_MAP.items()
        ]
    )



import re
import time
from xml.etree.ElementTree import Element, SubElement
from docmaptools import parse as docmap_parse
from jatsgenerator import build as jats_build
from elifetools.utils import doi_to_doi_uri
from elifecleaner import LOGGER

# for each ISSN, values for journal-id-type tag text
ISSN_JOURNAL_ID_MAP = {
    "2050-084X": {
        "nlm-ta": "elife",
        "hwp": "eLife",
        "publisher-id": "eLife",
    }
}

# for each ISSN, values for other journal metadata
ISSN_JOURNAL_META_MAP = {
    "2050-084X": {
        "journal-title": "eLife",
        "publisher-name": "eLife Sciences Publications, Ltd",
    }
}


def yield_journal_id_tags(root, journal_id_types):
    "find journal-id tags with matched journal-id-type attribute"
    for journal_id_tag in root.findall("./front/journal-meta/journal-id"):
        if (
            journal_id_tag.get("journal-id-type")
            and journal_id_tag.get("journal-id-type") in journal_id_types
        ):
            yield journal_id_tag


def is_xml_prc(root):
    "check if the XML is PRC format by comparing journal-id tag text for a mismatch"
    issn_tag = root.find("./front/journal-meta/issn")
    if issn_tag is not None and issn_tag.text in ISSN_JOURNAL_ID_MAP:
        journal_id_type_map = ISSN_JOURNAL_ID_MAP.get(issn_tag.text)
        # check if any of the journal-id tag values do not match the expected values
        for journal_id_tag in yield_journal_id_tags(root, journal_id_type_map.keys()):
            if journal_id_tag.text != journal_id_type_map.get(
                journal_id_tag.get("journal-id-type")
            ):
                return True
    # also check the elocation-id tag value if the journal meta has already been changed
    elocation_id_tag = root.find(".//front/article-meta/elocation-id")
    if elocation_id_tag is not None:
        if elocation_id_tag.text and elocation_id_tag.text.startswith(
            ELOCATION_ID_PRC_TERM
        ):
            return True
    return False


def transform_journal_id_tags(root, identifier=None):
    "replace file name tags in xml Element with names from file transformations list"
    issn_tag = root.find("./front/journal-meta/issn")
    if issn_tag is not None and issn_tag.text in ISSN_JOURNAL_ID_MAP:
        journal_id_type_map = ISSN_JOURNAL_ID_MAP.get(issn_tag.text)
        for journal_id_tag in yield_journal_id_tags(root, journal_id_type_map.keys()):
            LOGGER.info(
                "%s replacing journal-id tag text of type %s to %s",
                identifier,
                journal_id_tag.get("journal-id-type"),
                journal_id_type_map.get(journal_id_tag.get("journal-id-type")),
            )

            journal_id_tag.text = journal_id_type_map.get(
                journal_id_tag.get("journal-id-type")
            )
    return root


def transform_journal_meta_tag(root, tag_name, tag_path, identifier=None):
    "replace the text value of a tag in the journal meta"
    issn_tag = root.find("./front/journal-meta/issn")
    if issn_tag is not None and issn_tag.text in ISSN_JOURNAL_META_MAP:
        journal_meta_map = ISSN_JOURNAL_META_MAP.get(issn_tag.text)
        journal_title_tag = root.find(tag_path)
        if journal_title_tag is not None and journal_meta_map.get(tag_name):
            LOGGER.info(
                "%s replacing %s tag text to %s",
                identifier,
                tag_name,
                journal_meta_map.get(tag_name),
            )
            journal_title_tag.text = journal_meta_map.get(tag_name)
    return root


def transform_journal_title_tag(root, identifier=None):
    "replace journal-title tag in xml Element with names from file transformations list"
    return transform_journal_meta_tag(
        root,
        "journal-title",
        "./front/journal-meta/journal-title-group/journal-title",
        identifier,
    )


def transform_publisher_name_tag(root, identifier=None):
    "replace publisher-name tag in xml Element with names from file transformations list"
    return transform_journal_meta_tag(
        root,
        "publisher-name",
        "./front/journal-meta/publisher/publisher-name",
        identifier,
    )


def add_prc_custom_meta_tags(root, identifier=None):
    "add custom-meta tag in custom-meta-group"
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return root
    custom_meta_group_tag = article_meta_tag.find("custom-meta-group")
    if custom_meta_group_tag is None:
        # add the custom-meta-group tag
        custom_meta_group_tag = SubElement(article_meta_tag, "custom-meta-group")
    # add the custom-meta tag
    custom_meta_tag = SubElement(custom_meta_group_tag, "custom-meta")
    custom_meta_tag.set("specific-use", "meta-only")
    meta_name_tag = SubElement(custom_meta_tag, "meta-name")
    meta_name_tag.text = "publishing-route"
    meta_value_tag = SubElement(custom_meta_tag, "meta-value")
    meta_value_tag.text = "prc"
    return root


ELOCATION_ID_MATCH_PATTERN = r"e(.*)"

ELOCATION_ID_PRC_TERM = "RP"

ELOCATION_ID_REPLACEMENT_PATTERN = r"%s\1" % ELOCATION_ID_PRC_TERM


def transform_elocation_id(
    root,
    from_pattern=ELOCATION_ID_MATCH_PATTERN,
    to_pattern=ELOCATION_ID_REPLACEMENT_PATTERN,
    identifier=None,
):
    "change the elocation-id tag text value"
    elocation_id_tag = root.find(".//front/article-meta/elocation-id")
    if elocation_id_tag is not None:
        match_pattern = re.compile(from_pattern)
        new_elocation_id = match_pattern.sub(
            to_pattern,
            elocation_id_tag.text,
        )
        if new_elocation_id != elocation_id_tag.text:
            LOGGER.info(
                "%s changing elocation-id value %s to %s",
                identifier,
                elocation_id_tag.text,
                new_elocation_id,
            )
            elocation_id_tag.text = new_elocation_id
    return root


def version_doi_from_docmap(docmap_string, identifier=None):
    "find the latest preprint DOI from docmap"
    doi = None
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return doi
    LOGGER.info("Get latest preprint data from the docmap")
    preprint_data = docmap_parse.docmap_latest_preprint(d_json)
    if not preprint_data:
        LOGGER.warning(
            "%s no preprint data was found in the docmap",
            identifier,
        )
        return doi
    LOGGER.info("Find the doi in the docmap preprint data")
    doi = preprint_data.get("doi")
    if not doi:
        LOGGER.warning(
            "%s did not find doi data in the docmap preprint data",
            identifier,
        )
        return doi
    LOGGER.info("Version DOI from the docmap: %s", doi)
    return doi


# maximum supported verison number to check for non-version DOI values
MAX_VERSION = 999


def next_version_doi(doi, identifier=None):
    "generate the next version DOI value"
    if not doi:
        return None
    doi_base, version = doi.rsplit(".", 1)
    # check for integer
    try:
        int(version)
    except ValueError:
        LOGGER.warning(
            "%s version from DOI could not be converted to int, version %s",
            identifier,
            version,
        )
        return None
    if int(version) > MAX_VERSION:
        LOGGER.warning(
            "%s failed to determine the version from DOI, version %s exceeds MAX_VERSION %s",
            identifier,
            version,
            MAX_VERSION,
        )
        return None
    next_version = int(version) + 1
    next_doi = "%s.%s" % (doi_base, next_version)
    LOGGER.info(
        "%s next version doi, from DOI %s, next DOI %s", identifier, doi, next_doi
    )
    return next_doi


def add_version_doi(root, doi, identifier=None):
    "add version article-id tag for the doi to article-meta tag"
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return root
    # add article-id tag
    article_id_tag = Element("article-id")
    article_id_tag.set("pub-id-type", "doi")
    article_id_tag.set("specific-use", "version")
    article_id_tag.text = doi
    # insert the new tag into the XML after the last article-id tag
    insert_index = 1
    for tag_index, tag in enumerate(article_meta_tag.findall("*")):
        if tag.tag == "article-id":
            insert_index = tag_index + 1
    article_meta_tag.insert(insert_index, article_id_tag)
    return root


def review_date_from_docmap(docmap_string, identifier=None):
    "find the under-review date for the first preprint from the docmap"
    date_string = None
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return date_string
    LOGGER.info("Get first under-review happened date from the docmap")
    date_string = docmap_parse.preprint_review_date(d_json)
    if not date_string:
        LOGGER.warning(
            "%s no under-review happened date was found in the docmap",
            identifier,
        )
    return date_string


def volume_from_docmap(docmap_string, identifier=None):
    "from the docmap calculate the volume"
    LOGGER.info("Parse docmap json")
    d_json = docmap_parse.docmap_json(docmap_string)
    if not d_json:
        LOGGER.warning(
            "%s parsing docmap returned None",
            identifier,
        )
        return None
    history_data = docmap_parse.docmap_preprint_history(d_json)
    if not history_data:
        LOGGER.warning(
            "%s no history data from the docmap",
            identifier,
        )
        return None
    return volume_from_history_data(history_data, identifier)


# year of first publication, used for calculating volume
START_YEAR = 2011


def volume_from_history_data(history_data, identifier=None):
    "from history data, calculate the volume from the first reviewed preprint"
    if not history_data:
        LOGGER.warning(
            "%s no history_data",
            identifier,
        )
        return None
    LOGGER.info(
        "%s get first reviewed-preprint history event from the history_data", identifier
    )
    reviewed_preprint_event_data = None
    for event_data in history_data:
        if event_data.get("type") == "reviewed-preprint":
            reviewed_preprint_event_data = event_data
            break
    if not reviewed_preprint_event_data:
        LOGGER.warning(
            "%s no reviewed-preprint event found",
            identifier,
        )
        return None
    if not reviewed_preprint_event_data.get("date"):
        LOGGER.warning(
            "%s first reviewed-preprint event has no published date",
            identifier,
        )
        return None

    # get the date from the event data
    date_struct = date_struct_from_string(reviewed_preprint_event_data.get("date"))
    if not date_struct:
        LOGGER.warning(
            "%s could not parse date from the reviewed-preprint event",
            identifier,
        )
        return None
    year = date_struct[0]
    return year - START_YEAR


def date_struct_from_string(date_string):
    "parse the date_string into time.struct_time"
    formats = ["%Y-%m-%dT%H:%M:%S.%f%z", "%Y-%m-%dT%H:%M:%S%z", "%Y-%m-%d"]
    for date_format in formats:
        try:
            return time.strptime(date_string, date_format)
        except ValueError:
            LOGGER.info(
                'unable to parse "%s" using format "%s"',
                date_string,
                date_format,
            )
    return None


def add_history_date(root, date_type, date_struct, identifier=None):
    "find or add the history tag and add a date to it"
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return root
    # look for an existing history tag
    history_tag = article_meta_tag.find("./history")
    # if no history tag, add one
    if history_tag is None:
        # insert the new tag into the XML after the elocation-id tag
        insert_index = 1
        for tag_index, tag in enumerate(article_meta_tag.findall("*")):
            if tag.tag == "elocation-id":
                insert_index = tag_index + 1
        history_tag = Element("history")
        article_meta_tag.insert(insert_index, history_tag)
    # add date tag to the history tag
    date_tag = SubElement(history_tag, "date")
    if date_type:
        date_tag.set("date-type", date_type)
    jats_build.set_dmy(date_tag, date_struct)
    return root


EVENT_DESC_PREPRINT = "This manuscript was published as a preprint."
EVENT_DESC_REVIEWED_PREPRINT = "This manuscript was published as a reviewed preprint."
EVENT_DESC_REVISED_PREPRINT = "The reviewed preprint was revised."


def add_pub_history(root, history_data, identifier=None):
    "add the pub-history tag and add event data to it"
    if not history_data:
        return root
    article_meta_tag = root.find(".//front/article-meta")
    if article_meta_tag is None:
        LOGGER.warning(
            "%s article-meta tag not found",
            identifier,
        )
        return root
    # look for an existing pub-history tag
    pub_history_tag = article_meta_tag.find("./pub-history")
    # if no pub-history tag, add one
    if pub_history_tag is None:
        # insert the new tag into the XML after the history or elocation-id tag
        insert_index = 1
        for tag_index, tag in enumerate(article_meta_tag.findall("*")):
            if tag.tag in "history":
                insert_index = tag_index + 1
                break
            if tag.tag == "elocation-id":
                insert_index = tag_index + 1
        pub_history_tag = Element("pub-history")
        article_meta_tag.insert(insert_index, pub_history_tag)
    # add event tags to the pub-history tag
    added_reviewed_preprint = None
    for history_event_data in history_data:
        event_tag = SubElement(pub_history_tag, "event")
        if history_event_data.get("type"):
            if history_event_data.get("type") == "preprint":
                event_desc_tag = SubElement(event_tag, "event-desc")
                event_desc_tag.text = EVENT_DESC_PREPRINT
            elif history_event_data.get("type") == "reviewed-preprint":
                event_desc_tag = SubElement(event_tag, "event-desc")
                if added_reviewed_preprint:
                    # already added one reviewed-preprint
                    event_desc_tag.text = EVENT_DESC_REVISED_PREPRINT
                else:
                    # adding the first reviewed-preprint
                    event_desc_tag.text = EVENT_DESC_REVIEWED_PREPRINT
                added_reviewed_preprint = True
        if history_event_data.get("date"):
            date_struct = date_struct_from_string(history_event_data.get("date"))
            date_tag = SubElement(event_tag, "date")
            if history_event_data.get("type"):
                date_tag.set("date-type", history_event_data.get("type"))
            jats_build.set_dmy(date_tag, date_struct)
            date_tag.set("iso-8601-date", time.strftime("%Y-%m-%d", date_struct))
            if history_event_data.get("doi"):
                self_uri_tag = SubElement(event_tag, "self-uri")
                if history_event_data.get("type"):
                    self_uri_tag.set("content-type", history_event_data.get("type"))
                self_uri_tag.set(
                    "{http://www.w3.org/1999/xlink}href",
                    doi_to_doi_uri(history_event_data.get("doi")),
                )
    return root



import io
import os
import sys
import unittest
import zipfile
from xml.etree import ElementTree
from elifetools import parseJATS as parser
from elifecleaner import LOGGER, configure_logging, transform
from elifecleaner.transform import ArticleZipFile, WELLCOME_FUNDING_STATEMENT
from tests.helpers import delete_files_in_folder, read_fixture


class TestArticleZipFile(unittest.TestCase):
    def test_instantiate(self):
        xml_name = "sub_folder/file.txt"
        zip_name = "file.txt"
        file_path = "local_folder/sub_folder/file.txt"
        from_file = ArticleZipFile(xml_name, zip_name, file_path)
        expected = 'ArticleZipFile("%s", "%s", "%s")' % (xml_name, zip_name, file_path)
        self.assertEqual(str(from_file), expected)


class TestTransform(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"
        self.output_dir = "tests/tmp_output"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])
        delete_files_in_folder(self.output_dir, filter_out=[".keepme"])

    def test_transform_ejp_zip(self):
        zip_file = "tests/test_data/30-01-2019-RA-eLife-45644.zip"
        zip_file_name = zip_file.split(os.sep)[-1]
        info_prefix = (
            "INFO elifecleaner:transform:transform_ejp_zip: %s"
        ) % zip_file_name
        transform_info_prefix = (
            "INFO elifecleaner:transform:code_file_transformations: %s"
        ) % zip_file_name
        code_file_zip_info_prefix = (
            "INFO elifecleaner:transform:code_file_zip: %s"
        ) % zip_file_name
        rewrite_info_prefix = (
            "INFO elifecleaner:transform:xml_rewrite_file_tags: %s"
        ) % zip_file_name
        write_info_prefix = (
            "INFO elifecleaner:transform:write_xml_file: %s"
        ) % zip_file_name
        transform_history_prefix = (
            "INFO elifecleaner:transform:transform_xml_history_tags: %s"
        ) % zip_file_name
        rezip_info_prefix = ("INFO elifecleaner:transform:rezip: %s") % zip_file_name
        expected_new_zip_file_path = os.path.join(self.output_dir, zip_file_name)
        new_zip_file_path = transform.transform_ejp_zip(
            zip_file, self.temp_dir, self.output_dir
        )
        self.assertEqual(new_zip_file_path, expected_new_zip_file_path)
        log_file_lines = []
        with open(self.log_file, "r") as open_file:
            for line in open_file:
                log_file_lines.append(line)

        self.assertEqual(log_file_lines[0], "%s starting to transform\n" % info_prefix)

        self.assertEqual(
            log_file_lines[1],
            "%s code_file_name: Figure 5source code 1.c\n" % transform_info_prefix,
        )
        self.assertEqual(
            log_file_lines[2],
            (
                '%s from_file: ArticleZipFile("Figure 5source code 1.c",'
                ' "30-01-2019-RA-eLife-45644/Figure 5source code 1.c",'
                ' "tests/tmp/30-01-2019-RA-eLife-45644/Figure 5source code 1.c")\n'
            )
            % transform_info_prefix,
        )

        self.assertEqual(
            log_file_lines[3],
            (
                '%s to_file: ArticleZipFile("Figure 5source code 1.c.zip",'
                ' "30-01-2019-RA-eLife-45644/Figure 5source code 1.c.zip",'
                ' "tests/tmp_output/Figure 5source code 1.c.zip")\n'
            )
            % transform_info_prefix,
        )
        self.assertEqual(
            log_file_lines[4],
            (
                '%s zipping from_file: ArticleZipFile("Figure 5source code 1.c", '
                '"30-01-2019-RA-eLife-45644/Figure 5source code 1.c", '
                '"tests/tmp/30-01-2019-RA-eLife-45644/Figure 5source code 1.c"), '
                'to_file: ArticleZipFile("Figure 5source code 1.c.zip", '
                '"30-01-2019-RA-eLife-45644/Figure 5source code 1.c.zip", '
                '"tests/tmp_output/Figure 5source code 1.c.zip")\n'
            )
            % code_file_zip_info_prefix,
        )

        self.assertEqual(
            log_file_lines[5],
            ("%s rewriting xml tags\n") % rewrite_info_prefix,
        )
        self.assertEqual(
            log_file_lines[6],
            (
                "%s writing xml to file"
                " tests/tmp/30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.xml\n"
            )
            % write_info_prefix,
        )

        self.assertEqual(
            log_file_lines[7],
            (
                "%s "
                "article_type research-article, display_channel ['Research Article']\n"
            )
            % transform_history_prefix,
        )
        self.assertEqual(
            log_file_lines[9],
            (
                "%s writing xml to file"
                " tests/tmp/30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.xml\n"
            )
            % write_info_prefix,
        )
        self.assertEqual(
            log_file_lines[10],
            ("%s writing new zip file tests/tmp_output/30-01-2019-RA-eLife-45644.zip\n")
            % rezip_info_prefix,
        )
        # check output directory contents
        output_dir_list = os.listdir(self.output_dir)
        self.assertTrue("30-01-2019-RA-eLife-45644.zip" in output_dir_list)
        self.assertTrue("Figure 5source code 1.c.zip" in output_dir_list)

        # check zip file contents
        expected_infolist_filenames = [
            "30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.pdf",
            "30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.xml",
            "30-01-2019-RA-eLife-45644/Answers for the eLife digest.docx",
            "30-01-2019-RA-eLife-45644/Appendix 1.docx",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 1.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 10.pdf",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 11.pdf",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 12.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 13.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 14.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 15.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 2.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 3.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 4.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 5.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 6.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 7.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 8.png",
            "30-01-2019-RA-eLife-45644/Appendix 1figure 9.png",
            "30-01-2019-RA-eLife-45644/Figure 1.tif",
            "30-01-2019-RA-eLife-45644/Figure 2.tif",
            "30-01-2019-RA-eLife-45644/Figure 3.png",
            "30-01-2019-RA-eLife-45644/Figure 4.svg",
            "30-01-2019-RA-eLife-45644/Figure 4source data 1.zip",
            "30-01-2019-RA-eLife-45644/Figure 5.png",
            "30-01-2019-RA-eLife-45644/Figure 5source code 1.c.zip",
            "30-01-2019-RA-eLife-45644/Figure 6.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 10_HorC.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 1_U crassus.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 2_U pictorum.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 3_M margaritifera.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 4_P auricularius.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 5_PesB.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 6_HavA.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 7_HavB.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 8_HavC.png",
            "30-01-2019-RA-eLife-45644/Figure 6figure supplement 9_HorB.png",
            "30-01-2019-RA-eLife-45644/Figure 6source data 1.pdf",
            "30-01-2019-RA-eLife-45644/Manuscript.docx",
            "30-01-2019-RA-eLife-45644/Potential striking image.tif",
            "30-01-2019-RA-eLife-45644/Table 2source data 1.xlsx",
            "30-01-2019-RA-eLife-45644/transparent_reporting_Sakalauskaite.docx",
        ]
        with zipfile.ZipFile(new_zip_file_path, "r") as open_zipfile:
            infolist = open_zipfile.infolist()
        infolist_filenames = sorted(
            [info.filename for info in infolist if info.filename != ".keepme"]
        )
        self.assertEqual(infolist_filenames, expected_infolist_filenames)

        # assertions on XML file contents
        with zipfile.ZipFile(new_zip_file_path, "r") as open_zipfile:
            article_xml = open_zipfile.read(
                "30-01-2019-RA-eLife-45644/30-01-2019-RA-eLife-45644.xml"
            )
        self.assertTrue(
            b"<upload_file_nm>Figure 5source code 1.c.zip</upload_file_nm>"
            in article_xml
        )
        self.assertTrue(
            b"<upload_file_nm>Figure 5source code 1.c</upload_file_nm>"
            not in article_xml
        )

        # assertions on code zip file contents
        with zipfile.ZipFile(new_zip_file_path, "r") as open_zipfile:
            code_zip = open_zipfile.read(
                "30-01-2019-RA-eLife-45644/Figure 5source code 1.c.zip"
            )
        file_like_object = io.BytesIO(code_zip)
        with zipfile.ZipFile(file_like_object, "r") as open_zipfile:
            infolist = open_zipfile.infolist()
        self.assertEqual(
            [info.filename for info in infolist],
            ["30-01-2019-RA-eLife-45644/Figure 5source code 1.c"],
        )
        # self.assertTrue(False)


class TestXmlElementToString(unittest.TestCase):
    def test_xml_element_to_string(self):
        if sys.version_info < (3, 8):
            # pre Python 3.8 tag attributes are automatically alphabetised
            article_tag_xml_string = (
                '<article article-type="research-article"'
                ' xmlns:xlink="http://www.w3.org/1999/xlink">'
            )
        else:
            article_tag_xml_string = (
                '<article xmlns:xlink="http://www.w3.org/1999/xlink"'
                ' article-type="research-article">'
            )
        xml_string = (
            "%s<front><article-meta><permissions>"
            '<license license-type="open-access"'
            ' xlink:href="http://creativecommons.org/licenses/by/4.0/"/>'
            "</permissions></article-meta></front>"
            "</article>" % article_tag_xml_string
        )
        root = ElementTree.fromstring(xml_string)
        expected = '<?xml version="1.0" ?>%s' % xml_string
        self.assertEqual(transform.xml_element_to_string(root), expected)


class TestCodeFileList(unittest.TestCase):
    def test_code_file_list(self):
        xml_string = read_fixture("code_file_list.xml")
        expected = read_fixture("code_file_list.py")
        root = ElementTree.fromstring(xml_string)
        code_files = transform.code_file_list(root)
        self.assertEqual(code_files, expected)


class TestFindInFileNameMap(unittest.TestCase):
    def test_find_in_file_name_map(self):
        file_name = "file_one.txt"
        asset_file_name = "zip_sub_folder/file_one.txt"
        file_path = "local_folder/zip_sub_folder/file_one.txt"
        file_name_map = {asset_file_name: file_path}
        expected = (asset_file_name, file_path)
        self.assertEqual(
            transform.find_in_file_name_map(file_name, file_name_map), expected
        )

    def test_find_in_file_name_map_not_found(self):
        file_name = "file_one.txt"
        file_name_map = {}
        expected = (None, None)
        self.assertEqual(
            transform.find_in_file_name_map(file_name, file_name_map), expected
        )


class TestZipCodeFile(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_zip_code_file(self):
        file_name = "main.c"
        from_file = ArticleZipFile(
            file_name, "zip_folder/%s" % file_name, "tests/test_data/%s" % file_name
        )
        expected = ArticleZipFile(
            "%s.zip" % file_name,
            "zip_folder/%s.zip" % file_name,
            "%s/%s.zip" % (self.temp_dir, file_name),
        )
        to_file = transform.zip_code_file(from_file, self.temp_dir)
        # compare ArticleZipFile representation by casting them to str
        self.assertEqual(str(to_file), str(expected))


class TestFromFileToFileMap(unittest.TestCase):
    def test_from_file_to_file_map(self):
        from_xml_name = "source.c"
        to_xml_name = "source.c.zip"
        from_file = ArticleZipFile(from_xml_name, None, None)
        to_file = ArticleZipFile(to_xml_name, None, None)
        file_transformations = [(from_file, to_file)]
        expected = {from_xml_name: to_xml_name}
        self.assertEqual(
            transform.from_file_to_file_map(file_transformations), expected
        )


class TestTransformSubjectTags(unittest.TestCase):
    "tests for transform.transform_subject_tags()"

    def test_transform_subject_tags(self):
        "test removing (VOR) from a value and removing duplicate subject tags"
        xml_string = (
            "<article>"
            "<front>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Research Article (VOR)</subject>"
            "</subj-group>"
            '<subj-group subj-group-type="heading">'
            "<subject>Epidemiology and Global Health</subject>"
            "</subj-group>"
            '<subj-group subj-group-type="heading">'
            "<subject>Epidemiology and Global Health</subject>"
            "</subj-group>"
            '<subj-group subj-group-type="heading">'
            "<subject>Medicine</subject>"
            "</subj-group>"
            '<subj-group subj-group-type="heading">'
            "<subject>Medicine</subject>"
            "</subj-group>"
            "</article-categories>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        expected = (
            b"<article>"
            b"<front>"
            b"<article-meta>"
            b"<article-categories>"
            b'<subj-group subj-group-type="display-channel">'
            b"<subject>Research Article</subject>"
            b"</subj-group>"
            b'<subj-group subj-group-type="heading">'
            b"<subject>Epidemiology and Global Health</subject>"
            b"</subj-group>"
            b'<subj-group subj-group-type="heading">'
            b"<subject>Medicine</subject>"
            b"</subj-group>"
            b"</article-categories>"
            b"</article-meta>"
            b"</front>"
            b"</article>"
        )
        transform.transform_subject_tags(root, None)
        self.assertEqual(ElementTree.tostring(root), expected)


class TestTransformKwdTags(unittest.TestCase):
    "tests for transform.transform_kwd_tags()"

    def test_transform_kwd_tags(self):
        "test removing duplicate kwd tags"
        xml_string = (
            "<article>"
            "<front>"
            "<article-meta>"
            '<kwd-group kwd-group-type="research-organism">'
            "<title>Research organism</title>"
            "<kwd>Human</kwd>"
            "<kwd>Human</kwd>"
            "<kwd>Other</kwd>"
            "<kwd>Other</kwd>"
            "</kwd-group>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        expected = (
            b"<article>"
            b"<front>"
            b"<article-meta>"
            b'<kwd-group kwd-group-type="research-organism">'
            b"<title>Research organism</title>"
            b"<kwd>Human</kwd>"
            b"<kwd>Other</kwd>"
            b"</kwd-group>"
            b"</article-meta>"
            b"</front>"
            b"</article>"
        )
        transform.transform_kwd_tags(root, None)
        self.assertEqual(ElementTree.tostring(root), expected)


class TestTransformXmlFileTags(unittest.TestCase):
    def test_transform_xml_file_tags(self):
        # populate an ElementTree
        xml_string = read_fixture("code_file_list.xml")
        root = ElementTree.fromstring(xml_string)
        # specify from file, to file transformations
        file_transformations = []
        test_data = [
            {
                "from_xml": "Figure 5source code 1.c",
                "to_xml": "Figure 5source code 1.c.zip",
            },
            {
                "from_xml": "Figure 5source code 2.c",
                "to_xml": "Figure 5source code 2.c.zip",
            },
        ]
        for data in test_data:
            from_file = ArticleZipFile(data.get("from_xml"), None, None)
            to_file = ArticleZipFile(data.get("to_xml"), None, None)
            file_transformations.append((from_file, to_file))
        # invoke the function
        root_output = transform.transform_xml_file_tags(root, file_transformations)
        # find the tag in the XML root returned which will have been altered
        upload_file_nm_tags = root_output.findall(
            "./front/article-meta/files/file/upload_file_nm"
        )
        # assert the XML text is different
        self.assertEqual(upload_file_nm_tags[1].text, test_data[0].get("to_xml"))
        self.assertEqual(upload_file_nm_tags[2].text, test_data[1].get("to_xml"))


class TestTransformXmlHistoryTags(unittest.TestCase):
    def test_transform_xml_history_tags_research_article(self):
        "research-article XML will be unchanged"
        # populate an ElementTree
        xml_string = (
            '<article article-type="research-article">'
            "<front>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Test</subject>"
            "</subj-group>"
            "</article-categories>"
            "<history>"
            "<date/>"
            "</history>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        soup = parser.parse_xml(xml_string)
        # invoke the function
        root_output = transform.transform_xml_history_tags(root, soup, "test.zip")
        # find the tag in the XML root returned which will have been altered
        expected = '<?xml version="1.0" ?>%s' % xml_string
        self.assertEqual(transform.xml_element_to_string(root_output), expected)

    def test_transform_xml_history_tags_correction_article(self):
        "test removing history tag from a correction article XML"
        # populate an ElementTree
        xml_string = (
            '<article article-type="correction">'
            "<front>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Correction</subject>"
            "</subj-group>"
            "</article-categories>"
            "<history>"
            "<date/>"
            "</history>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        soup = parser.parse_xml(xml_string)
        # invoke the function
        root_output = transform.transform_xml_history_tags(root, soup, "test.zip")
        # find the tag in the XML root returned which will have been altered
        expected = (
            '<?xml version="1.0" ?>'
            '<article article-type="correction">'
            "<front>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Correction</subject>"
            "</subj-group>"
            "</article-categories>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        self.assertEqual(transform.xml_element_to_string(root_output), expected)

    def test_transform_xml_history_tags_prc_article(self):
        "test removing history tag from a Publish Review Curate (PRC) article"
        # populate an ElementTree
        xml_string = (
            '<article article-type="research-article">'
            "<front>"
            "<journal-meta>"
            '<journal-id journal-id-type="nlm-ta">__not_elife__</journal-id>'
            '<issn publication-format="electronic" pub-type="epub">2050-084X</issn>'
            "</journal-meta>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Test</subject>"
            "</subj-group>"
            "</article-categories>"
            "<history>"
            "<date/>"
            "</history>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        root = ElementTree.fromstring(xml_string)
        soup = parser.parse_xml(xml_string)
        # invoke the function
        root_output = transform.transform_xml_history_tags(root, soup, "test.zip")
        # find the tag in the XML root returned which will have been altered
        expected = (
            '<?xml version="1.0" ?>'
            '<article article-type="research-article">'
            "<front>"
            '<journal-meta><journal-id journal-id-type="nlm-ta">__not_elife__</journal-id>'
            '<issn publication-format="electronic" pub-type="epub">2050-084X</issn>'
            "</journal-meta>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Test</subject>"
            "</subj-group>"
            "</article-categories>"
            "</article-meta>"
            "</front>"
            "</article>"
        )
        self.assertEqual(transform.xml_element_to_string(root_output), expected)

    def test_keep_sent_for_review_date(self):
        "keep the sent-for-review date in the history tag"
        # populate an ElementTree
        sent_for_review_xml = '<history><date date-type="sent-for-review"><day>15</day><month>04</month><year>2023</year></date></history>'
        xml_string = (
            '<article article-type="research-article">'
            "<front>"
            "<journal-meta>"
            '<journal-id journal-id-type="nlm-ta">__not_elife__</journal-id>'
            '<issn publication-format="electronic" pub-type="epub">2050-084X</issn>'
            "</journal-meta>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Test</subject>"
            "</subj-group>"
            "</article-categories>"
            "<history>"
            '<date date-type="accepted">'
            "<day>16</day>"
            "<month>05</month>"
            "<year>2023</year>"
            "</date>"
            '<date date-type="received">'
            "<day>22</day>"
            "<month>12</month>"
            "<year>2022</year>"
            "</date>"
            '<date date-type="rev-recd">'
            "<day>11</day>"
            "<month>04</month>"
            "<year>2023</year>"
            "</date>"
            "%s"
            "</history>"
            "</article-meta>"
            "</front>"
            "</article>"
        ) % sent_for_review_xml
        root = ElementTree.fromstring(xml_string)
        soup = parser.parse_xml(xml_string)
        # invoke the function
        root_output = transform.transform_xml_history_tags(root, soup, "test.zip")
        # find the tag in the XML root returned which will have been altered
        expected = (
            '<?xml version="1.0" ?>'
            '<article article-type="research-article">'
            "<front>"
            '<journal-meta><journal-id journal-id-type="nlm-ta">__not_elife__</journal-id>'
            '<issn publication-format="electronic" pub-type="epub">2050-084X</issn>'
            "</journal-meta>"
            "<article-meta>"
            "<article-categories>"
            '<subj-group subj-group-type="display-channel">'
            "<subject>Test</subject>"
            "</subj-group>"
            "</article-categories>"
            "<history>%s</history>"
            "</article-meta>"
            "</front>"
            "</article>"
        ) % sent_for_review_xml
        self.assertEqual(transform.xml_element_to_string(root_output), expected)


class TestTransformXmlFunding(unittest.TestCase):
    def setUp(self):
        # XML prior to the funding-statement tag
        self.xml_string_start = (
            '<article article-type="research-article">'
            "<front>"
            "<article-meta>"
            "<funding-group>"
            "<award-group>"
            "<principal-award-recipient>author-99706</principal-award-recipient>"
            "<funding-source>Rosetrees Trust and Stoneygate Trust</funding-source>"
            "<award-id/>"
            "<principal-award-recipient>author-2458</principal-award-recipient>"
            "<funding-source>Rosetrees Trust and Stoneygate Trust</funding-source>"
            "<award-id/>"
            "<principal-award-recipient>author-2458</principal-award-recipient>"
            '<funding-source id="http://dx.doi.org/10.13039/100010269">Wellcome Trust (WT)</funding-source>'
            "<award-id/>"
            "<principal-award-recipient>author-2458</principal-award-recipient>"
            "<funding-source>European Research Council</funding-source>"
            "<award-id/>"
        )

        # XML following the funding-statement tag
        self.xml_string_end = (
            "</award-group>"
            "</funding-group>"
            "</article-meta>"
            "</front>"
            "</article>"
        )

    def test_transform_xml_funding(self):
        "test adding to the funding statement if a Wellcome funder is included"

        # populate an ElementTree
        funding_statement_xml = (
            "<funding-statement>The funders had no role in study design, data collection and"
            " interpretation, or the decision to submit the work for publication."
            "</funding-statement>"
        )
        xml_string = "%s%s%s" % (
            self.xml_string_start,
            funding_statement_xml,
            self.xml_string_end,
        )

        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = transform.transform_xml_funding(root, "test.zip")
        # confirm XML root returned is modified
        self.assertTrue(
            WELLCOME_FUNDING_STATEMENT in transform.xml_element_to_string(root_output)
        )

    def test_transform_xml_funding_sentence_exists(self):
        "test adding to the funding statement if a Wellcome funder is included"

        # populate an ElementTree
        funding_statement_xml = (
            "<funding-statement>The funders had no role in study design, data collection and"
            " interpretation, or the decision to submit the"
            " work for publication. %s</funding-statement>" % WELLCOME_FUNDING_STATEMENT
        )
        xml_string = "%s%s%s" % (
            self.xml_string_start,
            funding_statement_xml,
            self.xml_string_end,
        )

        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = transform.transform_xml_funding(root, "test.zip")
        # confirm XML root returned is modified
        self.assertTrue(
            ("%s</funding-statement>" % WELLCOME_FUNDING_STATEMENT)
            in transform.xml_element_to_string(root_output)
        )

    def test_transform_xml_funding_tag_missing(self):
        "test if the funding-statement tag is missing"

        # populate an ElementTree
        xml_string = "%s%s" % (
            self.xml_string_start,
            self.xml_string_end,
        )
        root = ElementTree.fromstring(xml_string)
        # invoke the function
        root_output = transform.transform_xml_funding(root, "test.zip")
        # confirm XML root returned is modified
        self.assertTrue(
            "<funding-statement>%s</funding-statement>" % WELLCOME_FUNDING_STATEMENT
            in transform.xml_element_to_string(root_output)
        )


class TestTransformAssetFileNameMap(unittest.TestCase):
    def test_transform_asset_file_name_map_empty(self):
        # test empty arguments
        asset_file_name_map = {}
        file_transformations = []
        expected = {}
        new_asset_file_name_map = transform.transform_asset_file_name_map(
            asset_file_name_map, file_transformations
        )
        self.assertEqual(new_asset_file_name_map, expected)

    def test_transform_asset_file_name_map(self):
        # test one file example
        asset_file_name_map = {"zip_folder/main.c": "local_folder/zip_folder/main.c"}
        from_file = ArticleZipFile(
            "main.c", "zip_folder/main.c", "local_folder/zip_folder/main.c"
        )
        to_file = ArticleZipFile(
            "main.c.zip", "zip_folder/main.c.zip", "tmp_folder/zip_folder/main.c.zip"
        )
        file_transformations = [(from_file, to_file)]
        expected = {"zip_folder/main.c.zip": "tmp_folder/zip_folder/main.c.zip"}
        new_asset_file_name_map = transform.transform_asset_file_name_map(
            asset_file_name_map, file_transformations
        )
        self.assertEqual(new_asset_file_name_map, expected)

    def test_transform_asset_file_name_map_mismatch(self):
        # test if from_file is not found in the name map
        asset_file_name_map = {}
        from_file = ArticleZipFile(
            "main.c", "zip_folder/main.c", "local_folder/zip_folder/main.c"
        )
        to_file = ArticleZipFile(
            "main.c.zip", "zip_folder/main.c.zip", "tmp_folder/zip_folder/main.c.zip"
        )
        file_transformations = [(from_file, to_file)]
        expected = {}
        new_asset_file_name_map = transform.transform_asset_file_name_map(
            asset_file_name_map, file_transformations
        )
        self.assertEqual(new_asset_file_name_map, expected)


class TestCreateZipFromFileMap(unittest.TestCase):
    def setUp(self):
        self.temp_dir = "tests/tmp"

    def tearDown(self):
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_create_zip_from_file_map(self):
        zip_path = os.path.join(self.temp_dir, "test.zip")
        # add a file from the test data
        file_name = "zip_folder/main.c"
        file_path = "tests/test_data/main.c"
        file_name_map = {file_name: file_path}
        transform.create_zip_from_file_map(zip_path, file_name_map)
        with zipfile.ZipFile(zip_path, "r") as open_zipfile:
            infolist = open_zipfile.infolist()
        self.assertEqual(infolist[0].filename, file_name)



import os
import unittest
from xml.etree import ElementTree
from xml.etree.ElementTree import Element
from elifetools import xmlio
from elifecleaner import configure_logging, fig, LOGGER
from tests.helpers import delete_files_in_folder, read_log_file_lines


class TestInfFileIdentifier(unittest.TestCase):
    "tests for fig.inf_file_identifier()"

    def test_inf_file_identifer(self):
        "identifier portion of an inline-graphic file name"
        inf_file_name = "elife-70493-inf1.png"
        expected = "inf1"
        self.assertEqual(fig.inf_file_identifier(inf_file_name), expected)


class TestFigFileNameIdentifer(unittest.TestCase):
    "tests for fig.fig_file_name_identifier()"

    def test_fig_file_name_identifier(self):
        "generate a fig graphic id value"
        sub_article_id = "sa2"
        fig_index = "1"
        expected = "sa2-fig1"
        self.assertEqual(
            fig.fig_file_name_identifier(sub_article_id, fig_index), expected
        )


class TestFigFileName(unittest.TestCase):
    "tests for fig.fig_file_name()"

    def test_fig_file_name(self):
        "generate a fig graphic file name from an inline-graphic file name"
        inf_file_name = "elife-70493-inf1.png"
        sub_article_id = "sa2"
        fig_index = "1"
        expected = "elife-70493-sa2-fig1.png"
        self.assertEqual(
            fig.fig_file_name(inf_file_name, sub_article_id, fig_index),
            expected,
        )


class TestMatchFigLabelContent(unittest.TestCase):
    "tests for fig.match_fig_label_content()"

    def test_match_fig_label_content(self):
        "test matching paragraph content as a fig label"
        cases = [
            (None, False),
            ("", False),
            ("Test", False),
            ("Review image 1.", True),
            ("Review image 1", True),
            ("Review table 1.", False),
            ("Author response image 1.", True),
        ]
        for content, expected in cases:
            self.assertEqual(fig.match_fig_label_content(content), expected)


class TestIsPLabel(unittest.TestCase):
    "tests for fig.is_p_label()"

    def test_is_p_label(self):
        "example of a matched label in a p tag"
        xml_string = "<p><bold>Review image 1.</bold></p>"
        expected = True
        self.assertEqual(
            fig.is_p_label(ElementTree.fromstring(xml_string), None, None, None),
            expected,
        )

    def test_false(self):
        "not a label in a p tag"
        xml_string = "<p>Foo.</p>"
        expected = False
        self.assertEqual(
            fig.is_p_label(ElementTree.fromstring(xml_string), None, None, None),
            expected,
        )


class TestIsPInlineGraphic(unittest.TestCase):
    "tests for fig.is_p_inline_graphic()"

    def test_is_p_inline_graphic(self):
        "simple example of an inline-graphic p tag"
        xml_string = "<p><inline-graphic /></p>"
        expected = True
        self.assertEqual(
            fig.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )

    def test_whitespace(self):
        "test if when whitespace surrounds the inline-graphic tag"
        xml_string = "<p>   <inline-graphic />      </p>"
        expected = True
        self.assertEqual(
            fig.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )

    def test_false(self):
        "not an inline-graphic p tag"
        xml_string = "<p>Foo.</p>"
        expected = False
        self.assertEqual(
            fig.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )

    def test_none(self):
        "an inline-graphic tag and other content returns None"
        xml_string = "<p>An <inline-graphic/></p>"
        expected = None
        self.assertEqual(
            fig.is_p_inline_graphic(
                ElementTree.fromstring(xml_string), None, None, None
            ),
            expected,
        )


class TestFigTagIndexGroups(unittest.TestCase):
    "tests for fig.fig_tag_index_groups()"

    def test_fig_tag_index_groups(self):
        "test finding a group of p tags to be converted to a fig"
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
            b"</body>"
        )
        expected = [{"label_index": 1, "caption_index": 2, "inline_graphic_index": 3}]
        result = fig.fig_tag_index_groups(
            ElementTree.fromstring(xml_string), None, None
        )
        self.assertEqual(len(result), 1)
        self.assertDictEqual(result[0], expected[0])

    def test_multiple_figs(self):
        "test multiple groups of p tags to be converted to fig tags"
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"<p><bold>Review image 2</bold></p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
            b"</body>"
        )
        expected = [
            {"label_index": 1, "caption_index": 2, "inline_graphic_index": 3},
            {"label_index": 4, "caption_index": False, "inline_graphic_index": 5},
        ]
        result = fig.fig_tag_index_groups(
            ElementTree.fromstring(xml_string), None, None
        )
        self.assertEqual(len(result), 2)
        # first fig
        self.assertDictEqual(result[0], expected[0])
        # second fig
        self.assertDictEqual(result[1], expected[1])

    def test_empty(self):
        "test if no p tag groups are found"
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"</body>"
        )
        result = fig.fig_tag_index_groups(
            ElementTree.fromstring(xml_string), None, None
        )
        self.assertEqual(len(result), 0)

    def test_none(self):
        "test None as input to finding p tag groups"
        result = fig.fig_tag_index_groups(None, None, None)
        self.assertEqual(result, [])


class TestTransformFigGroups(unittest.TestCase):
    "tests for fig.transform_fig_groups()"

    def test_transform_fig_groups(self):
        "give body tag and indexes of p tags to transform to a fig tag"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"<p><bold>Review image 2</bold></p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
            b"</body>"
        )
        body_tag = ElementTree.fromstring(xml_string)
        fig_index_groups = [
            {"label_index": 1, "caption_index": 2, "inline_graphic_index": 3},
            {"label_index": 4, "caption_index": False, "inline_graphic_index": 5},
        ]
        sub_article_id = "sa1"
        expected = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b'<fig id="sa1fig1">'
            b"<label>Review image 1.</label>"
            b"<caption>"
            b"<title>"
            b"This is the caption for this image that describes what it contains."
            b"</title>"
            b"</caption>"
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="elife-70493-sa1-fig1.png" />'
            b"</fig>"
            b'<fig id="sa1fig2">'
            b"<label>Review image 2</label>"
            b'<graphic mimetype="image" mime-subtype="jpg" xlink:href="elife-70493-sa1-fig2.jpg" />'
            b"</fig>"
            b"</body>"
        )
        fig.transform_fig_groups(body_tag, fig_index_groups, sub_article_id)
        self.assertEqual(ElementTree.tostring(body_tag), expected)


class TestStripTagText(unittest.TestCase):
    "tests for fig.strip_tag_text()"

    def test_strip_tag_text(self):
        "test an example with whitespace to the left of the tag"
        xml_string = b"<p> <inline-graphic /> </p>"
        expected = b"<p><inline-graphic /> </p>"
        p_tag = ElementTree.fromstring(xml_string)
        fig.strip_tag_text(p_tag)
        self.assertEqual(ElementTree.tostring(p_tag), expected)

    def test_non_element(self):
        "test if the input is not a tag"
        tag = "foo"
        self.assertEqual(fig.strip_tag_text(tag), None)


class TestStripTagTail(unittest.TestCase):
    "tests for fig.strip_tag_tail()"

    def test_strip_tag_tail(self):
        "test an example with whitespace to the left and right of the tag"
        xml_string = b"<p> <inline-graphic /> </p>"
        expected = b"<inline-graphic />"
        p_tag = ElementTree.fromstring(xml_string)
        inline_graphic_tag = p_tag.find("inline-graphic")
        fig.strip_tag_tail(inline_graphic_tag)
        self.assertEqual(ElementTree.tostring(inline_graphic_tag), expected)

    def test_non_element(self):
        "test if the input is not a tag"
        tag = "foo"
        self.assertEqual(fig.strip_tag_tail(tag), None)


class TestRemoveTagAttributes(unittest.TestCase):
    "tests for fig.remove_tag_attributes()"

    def test_remove_tag_attributes(self):
        "test cleaning attributes from a p tag"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = (
            b'<p xmlns:xlink="http://www.w3.org/1999/xlink" id="p" xlink:href="foo" />'
        )
        tag = ElementTree.fromstring(xml_string)
        expected = b"<p />"
        fig.remove_tag_attributes(tag)
        self.assertEqual(ElementTree.tostring(tag), expected)

    def test_non_element(self):
        "test if the input is not a tag"
        tag = "foo"
        self.assertEqual(fig.remove_tag_attributes(tag), None)


class TestInlineGraphicTagFromTag(unittest.TestCase):
    "tests for fig.inline_graphic_tag_from_tag()"

    def test_tag_found(self):
        "test getting a inline-graphic tag from a p tag"
        xml_string = b"<p>   <inline-graphic />   </p>"
        tag = ElementTree.fromstring(xml_string)
        expected = b"<inline-graphic />"
        result = fig.inline_graphic_tag_from_tag(tag)
        self.assertEqual(ElementTree.tostring(result), expected)


class TestInlineGraphicHrefs(unittest.TestCase):
    "tests for fig.inline_graphic_hrefs()"

    def test_inline_graphic_hrefs(self):
        "get a list of xlink:href values from inline-graphic tags to be converted to fig"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body>"
            b"<p><bold>Review image 1.</bold></p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf1.png"/></p>'
            b"<p>Next paragraph is not an inline-graphic href.</p>"
            b'<p><inline-graphic xlink:href="elife-70493-inf2.png"/></p>'
            b"</body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-70493-inf1.png"]
        result = fig.inline_graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestGraphicHrefs(unittest.TestCase):
    "tests for fig.graphic_hrefs()"

    def test_graphic_hrefs(self):
        "get a list of xlink:href values from graphic tags"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink"><body>'
            b"<p>"
            b'<fig id="sa1fig1">'
            b"<label>Review image 1.</label>"
            b"<caption>"
            b"<title>A title.</title>"
            b"<p>A caption.</p>"
            b"</caption>"
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="elife-70493-sa1-fig1.png" />'
            b"</fig>"
            b"</p>"
            b"</body></sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = ["elife-70493-sa1-fig1.png"]
        result = fig.graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)

    def test_graphic_hrefs_no_match(self):
        "empty list of xlink:href values when there is no graphic tag"
        xml_string = (
            b'<sub-article id="sa1" xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<body><p/></body>"
            b"</sub-article>"
        )
        identifier = "test.zip"
        tag = ElementTree.fromstring(xml_string)
        expected = []
        result = fig.graphic_hrefs(tag, identifier)
        self.assertEqual(result, expected)


class TestSetLabelTag(unittest.TestCase):
    "tests for fig.set_label_tag()"

    def test_set_label_tag(self):
        "create label tag for a fig"
        body_tag_xml_string = b"<body><p><bold>Label</bold></p></body>"
        body_tag = ElementTree.fromstring(body_tag_xml_string)
        tag = Element("fig")
        label_index = 0
        expected = b"<fig><label>Label</label></fig>"
        fig.set_label_tag(tag, body_tag, label_index)
        self.assertEqual(ElementTree.tostring(tag), expected)


class TestSplitTtitleParts(unittest.TestCase):
    "tests for fig.split_title_parts()"

    def test_split_title_parts(self):
        "test spliting a simple example"
        xml_string = "<p>A title. A caption paragraph.</p>"
        expected = ["<p>A title.", " A caption paragraph.", "</p>"]
        result = fig.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_multiple_sentences(self):
        "test spliting a simple example"
        xml_string = "<p>A title. A caption paragraph. Another paragraph.</p>"
        expected = [
            "<p>A title.",
            " A caption paragraph.",
            " Another paragraph.",
            "</p>",
        ]
        result = fig.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_namespace(self):
        "test spliting a simple example"
        xml_string = (
            '<p xmlns:xlink="http://www.w3.org/1999/xlink">'
            'A title only (<ext-link xlink:href="http://example.org"/>).'
            "</p>"
        )
        expected = [
            '<p xmlns:xlink="http://www.w3.org/1999/xlink">A title only (<ext-link '
            'xlink:href="http://example.org"/>).',
            "</p>",
        ]
        result = fig.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_unmatched_tags(self):
        "example where there are unmatched tags in the result"
        xml_string = xml_string = (
            "<p>The title<bold>.</bold> Another <bold>bold term</bold>."
            " Another paragraph.</p>"
        )
        expected = [
            "<p>The title<bold>.",
            "</bold> Another <bold>bold term</bold>.",
            " Another paragraph.",
            "</p>",
        ]
        result = fig.split_title_parts(xml_string)
        self.assertEqual(result, expected)

    def test_blank_string(self):
        "test splitting a blank string"
        xml_string = ""
        expected = []
        result = fig.split_title_parts(xml_string)
        self.assertEqual(result, expected)


class TestTitleParagraphContent(unittest.TestCase):
    "tests for fig.title_paragraph_content()"

    def test_title_paragraph_content(self):
        "a simple example of string parts"
        string_list = [
            "<p>Caption title.",
            " Caption paragraph.",
            "</p>",
        ]
        expected = ("<p>Caption title.", " Caption paragraph.</p>")
        result = fig.title_paragraph_content(string_list)
        self.assertEqual(result, expected)

    def test_multiple_sentences(self):
        "a simple example of string parts"
        string_list = [
            "<p>A title.",
            " A caption paragraph.",
            " Another paragraph.",
            "</p>",
        ]
        expected = ("<p>A title.", " A caption paragraph. Another paragraph.</p>")
        result = fig.title_paragraph_content(string_list)
        self.assertEqual(result, expected)

    def test_italic(self):
        "test where an unmatched inline formatting tag is repaired"
        string_list = [
            "<p>The title<bold>.",
            "</bold> Another <bold>bold term</bold>.",
            " Another paragraph.",
            "</p>",
        ]
        expected = (
            "<p>The title<bold>.</bold> Another <bold>bold term</bold>.",
            " Another paragraph.</p>",
        )
        result = fig.title_paragraph_content(string_list)
        self.assertEqual(result, expected)

    def test_empty_list(self):
        "test an empty list"
        string_list = [""]
        expected = ("", None)
        result = fig.title_paragraph_content(string_list)
        self.assertEqual(result, expected)


class TestCaptionTitleParagraph(unittest.TestCase):
    "tests for fig.caption_title_paragraph()"

    def test_simple_title(self):
        "test a simple example"
        xml_string = "<p>Title. Caption.</p>"
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = fig.caption_title_paragraph(tag)
        self.assertEqual(ElementTree.tostring(caption_title_p_tag), b"<p>Title.</p>")
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag), b"<p>Caption.</p>"
        )

    def test_organism_title(self):
        "test for italic tag included"
        xml_string = "<p>In <italic>B. subtilis</italic>, the title. Caption.</p>"
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = fig.caption_title_paragraph(tag)
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b"<p>In <italic>B. subtilis</italic>, the title.</p>",
        )
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag), b"<p>Caption.</p>"
        )

    def test_two_bold_tags(self):
        "edge case with more than one bold tag and second one is around the title full stop"
        xml_string = (
            "<p>The title<bold>.</bold> Another <bold>bold term</bold>."
            " Another paragraph.</p>"
        )
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = fig.caption_title_paragraph(tag)
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b"<p>The title<bold>.</bold> Another <bold>bold term</bold>.</p>",
        )
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag), b"<p>Another paragraph.</p>"
        )

    def test_ext_link_tag(self):
        "edge case with an ext-link tag in the title and challenging full stop separations"
        xml_string = '<p xmlns:xlink="http://www.w3.org/1999/xlink">(Figure 2A from <ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>)<bold>.</bold> Comparison of Tests against Γ ( 4,5) (a = 0.05). The normality tests used were severely underpowered for n&lt;100 (<ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>).</p>'
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = fig.caption_title_paragraph(tag)
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b'<p xmlns:xlink="http://www.w3.org/1999/xlink">(Figure 2A from <ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>)<bold>.</bold> Comparison of Tests against &#915; ( 4,5) (a = 0.</p>',
        )
        self.assertEqual(
            ElementTree.tostring(caption_paragraph_p_tag),
            b'<p xmlns:xlink="http://www.w3.org/1999/xlink">05). The normality tests used were severely underpowered for n&lt;100 (<ext-link ext-link-type="uri" xlink:href="https://example.org/one/two">(Anonymous et al., 2011)</ext-link>).</p>',
        )

    def test_mml_tag(self):
        "edge case where a full stop in math formula should not be used as separate parts"
        xml_string = '<p xmlns:mml="http://www.w3.org/1998/Math/MathML">For one participant <inline-formula><mml:math alttext="" display="inline"><mml:mspace width="0.222em" /></mml:math></inline-formula> is a formula.</p>'
        tag = ElementTree.fromstring(xml_string)
        caption_title_p_tag, caption_paragraph_p_tag = fig.caption_title_paragraph(tag)
        self.assertEqual(
            ElementTree.tostring(caption_title_p_tag),
            b'<p xmlns:mml="http://www.w3.org/1998/Math/MathML">For one participant <inline-formula><mml:math alttext="" display="inline"><mml:mspace width="0.222em" /></mml:math></inline-formula> is a formula.</p>',
        )
        self.assertEqual(ElementTree.tostring(caption_paragraph_p_tag), b"<p />")


class TestSetCaptionTag(unittest.TestCase):
    "tests for fig.set_caption_tag()"

    def test_set_caption_tag(self):
        "create caption tag for a fig"
        body_tag_xml_string = (
            b"<body>"
            b"<p>Title <italic>here</italic>. "
            b"Caption paragraph <italic>ici</italic>."
            b"</p>"
            b"</body>"
        )
        body_tag = ElementTree.fromstring(body_tag_xml_string)
        tag = Element("fig")
        caption_index = 0
        expected = (
            b"<fig>"
            b"<caption>"
            b"<title>Title <italic>here</italic>.</title>"
            b"<p>Caption paragraph <italic>ici</italic>.</p>"
            b"</caption>"
            b"</fig>"
        )
        fig.set_caption_tag(tag, body_tag, caption_index)
        self.assertEqual(ElementTree.tostring(tag), expected)


class TestSetGraphicTag(unittest.TestCase):
    "tests for fig.set_graphic_tag()"

    def test_set_graphic_tag(self):
        "create graphic tag for a fig"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = '<fig xmlns:xlink="http://www.w3.org/1999/xlink" />'
        tag = ElementTree.fromstring(xml_string)
        image_href = "image.png"
        new_file_name = "new_image.png"
        expected = (
            b'<fig xmlns:xlink="http://www.w3.org/1999/xlink">'
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="new_image.png" />'
            b"</fig>"
        )
        fig.set_graphic_tag(tag, image_href, new_file_name)
        self.assertEqual(ElementTree.tostring(tag), expected)


class TestTransformFigGroup(unittest.TestCase):
    "tests for fig.transform_fig_group()"

    def test_transform_fig_group(self):
        "give body tag and indexes of p tags to transform to a fig tag"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b"<p><bold>Review image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
            b"</body>"
        )
        body_tag = ElementTree.fromstring(xml_string)
        fig_index = 1
        fig_group = {"label_index": 1, "caption_index": 2, "inline_graphic_index": 3}
        sub_article_id = "sa1"
        expected = (
            b'<body xmlns:xlink="http://www.w3.org/1999/xlink">'
            b"<p>First paragraph.</p>"
            b'<fig id="sa1fig1">'
            b"<label>Review image 1.</label>"
            b"<caption>"
            b"<title>"
            b"This is the caption for this image that describes what it contains."
            b"</title>"
            b"</caption>"
            b'<graphic mimetype="image" mime-subtype="png" xlink:href="elife-70493-sa1-fig1.png" />'
            b"</fig>"
            b"</body>"
        )

        fig.transform_fig_group(body_tag, fig_index, fig_group, sub_article_id)
        self.assertEqual(ElementTree.tostring(body_tag), expected)


def sub_article_xml_fixture(
    article_type, sub_article_id, front_stub_xml_string, body_xml_string
):
    "generate test fixture XML for a sub-article"
    return (
        b'<sub-article xmlns:xlink="http://www.w3.org/1999/xlink" '
        b'article-type="%s" id="%s">%s%s</sub-article>'
        % (article_type, sub_article_id, front_stub_xml_string, body_xml_string)
    )


def sa1_sub_article_xml_fixture():
    "generate XML test fixture for a referee-report"
    article_type = b"referee-report"
    sub_article_id = b"sa1"
    front_stub_xml_string = (
        b"<front-stub>"
        b'<article-id pub-id-type="doi">10.7554/eLife.79713.1.sa1</article-id>'
        b"<title-group>"
        b"<article-title>Reviewer #1 (Public Review):</article-title>"
        b"</title-group>"
        b"</front-stub>"
    )
    body_xml_string = (
        b"<body>"
        b"<p>First paragraph.</p>"
        b"<p><bold>Review image 1.</bold></p>"
        b"<p>This is the caption for this image that describes what it contains.</p>"
        b'<p> <inline-graphic xlink:href="elife-70493-inf1.png" /> </p>'
        b"<p>Another paragraph with an inline graphic "
        b'<inline-graphic xlink:href="elife-70493-inf2.jpg" /></p>'
        b"</body>"
    )
    return sub_article_xml_fixture(
        article_type, sub_article_id, front_stub_xml_string, body_xml_string
    )


def sa4_sub_article_xml_fixture(body_xml_string):
    "generate XML test fixture for an author-comment"
    article_type = b"author-comment"
    sub_article_id = b"sa4"
    front_stub_xml_string = (
        b"<front-stub>"
        b'<article-id pub-id-type="doi">10.7554/eLife.79713.1.sa4</article-id>'
        b"<title-group><article-title>Author response</article-title></title-group>"
        b"</front-stub>"
    )
    return sub_article_xml_fixture(
        article_type, sub_article_id, front_stub_xml_string, body_xml_string
    )


class TestTransformFig(unittest.TestCase):
    "tests for fig.transform_fig()"

    def setUp(self):
        self.identifier = "test.zip"
        self.temp_dir = "tests/tmp"
        self.log_file = os.path.join(self.temp_dir, "test.log")
        self.log_handler = configure_logging(self.log_file)

    def tearDown(self):
        LOGGER.removeHandler(self.log_handler)
        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])

    def test_referee_report(self):
        "convert tags to a fig in a referee-report fixture"
        # register XML namespaces
        xmlio.register_xmlns()
        xml_string = sa1_sub_article_xml_fixture()
        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue("<p>First paragraph.</p>" in rough_xml_string.decode("utf8"))
        self.assertTrue(
            '<inline-graphic xlink:href="elife-70493-inf2.jpg" />'
            in rough_xml_string.decode("utf8")
        )
        print(rough_xml_string.decode("utf8"))
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa1fig1">'
                "<label>Review image 1.</label>"
                "<caption>"
                "<title>"
                "This is the caption for this image that describes what it contains."
                "</title>"
                "</caption>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa1-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf2.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )
        info_prefix = "INFO elifecleaner:fig"
        expected = [
            '%s:%s: %s potential label "Review image 1." in p tag 1 of id sa1\n'
            % (info_prefix, "is_p_label", self.identifier),
            "%s:%s: %s label p tag index 1 of id sa1\n"
            % (info_prefix, "fig_tag_index_groups", self.identifier),
            "%s:%s: %s no inline-graphic tag found in p tag 2 of id sa1\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
            "%s:%s: %s only inline-graphic tag found in p tag 3 of id sa1\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
        ]
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    def test_author_response(self):
        "convert tags to a fig in an author-comment fixture"
        # register XML namespaces
        xmlio.register_xmlns()
        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b"<p>This is the caption for this image that describes what it contains.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue("<p>First paragraph.</p>" in rough_xml_string.decode("utf8"))
        self.assertTrue(
            '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
            in rough_xml_string.decode("utf8")
        )
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa4fig1">'
                "<label>Author response image 1.</label>"
                "<caption>"
                "<title>"
                "This is the caption for this image that describes what it contains."
                "</title>"
                "</caption>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa4-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )
        # self.assertTrue(False)
        info_prefix = "INFO elifecleaner:fig"
        expected = [
            '%s:%s: %s potential label "Author response image 1." in p tag 1 of id sa4\n'
            % (info_prefix, "is_p_label", self.identifier),
            "%s:%s: %s label p tag index 1 of id sa4\n"
            % (info_prefix, "fig_tag_index_groups", self.identifier),
            "%s:%s: %s no inline-graphic tag found in p tag 2 of id sa4\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
            "%s:%s: %s only inline-graphic tag found in p tag 3 of id sa4\n"
            % (info_prefix, "is_p_inline_graphic", self.identifier),
        ]
        self.assertEqual(read_log_file_lines(self.log_file), expected)

    def test_caption_italic(self):
        "convert tags to a fig where the caption has an inline intalic tag"
        # register XML namespaces
        xmlio.register_xmlns()

        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b'<p>Caption with an <italic id="foo">italic</italic> tag or '
            b"<bold>bold</bold> too.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )

        xml_string = sa4_sub_article_xml_fixture(body_xml_string)
        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa4fig1">'
                "<label>Author response image 1.</label>"
                "<caption>"
                "<title>"
                'Caption with an <italic id="foo">italic</italic> tag or <bold>bold</bold> too.'
                "</title>"
                "</caption>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa4-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )

    def test_no_caption(self):
        "convert tags to a fig where there is no p tag for the caption content"
        # register XML namespaces
        xmlio.register_xmlns()
        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertTrue(
            (
                "<body>"
                "<p>First paragraph.</p>"
                '<fig id="sa4fig1">'
                "<label>Author response image 1.</label>"
                '<graphic mimetype="image" mime-subtype="png" '
                'xlink:href="elife-70493-sa4-fig1.png" />'
                "</fig>"
                "<p>Another paragraph with an inline graphic "
                '<inline-graphic xlink:href="elife-70493-inf4.jpg" />'
                "</p>"
                "</body>"
            )
            in rough_xml_string.decode("utf8")
        )

    def test_multiple_caption(self):
        "test if there are multiple caption p tags before the inline-graphic tag"
        # register XML namespaces
        xmlio.register_xmlns()
        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b"<p>Caption paragraph one.</p>"
            b"<p>Caption paragraph two.</p>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, rough_xml_string)

    def test_non_p_tag(self):
        "test if the p tag order is disrupted with a sec tag"
        # register XML namespaces
        xmlio.register_xmlns()

        body_xml_string = (
            b"<body>"
            b"<p>First paragraph.</p>"
            b"<p><bold>Author response image 1.</bold></p>"
            b"<sec>Non-p tag</sec>"
            b'<p> <inline-graphic xlink:href="elife-70493-inf3.png" /> </p>'
            b"<p>Another paragraph with an inline graphic "
            b'<inline-graphic xlink:href="elife-70493-inf4.jpg" /></p>'
            b"</body>"
        )
        xml_string = sa4_sub_article_xml_fixture(body_xml_string)

        sub_article_root = ElementTree.fromstring(xml_string)
        root = fig.transform_fig(sub_article_root, self.identifier)

        rough_xml_string = ElementTree.tostring(root, "utf-8")
        self.assertEqual(xml_string, rough_xml_string)
